import { ReflectiveInjector, OnInit, OnDestroy } from "@angular/core";
import { Nav, Events, Platform } from "ionic-angular";
import { DataProvider } from "../providers/data-provider";
import { Slots } from "../utils/slots";

/**
 * Use with VirtualScroll
 */
export abstract class ListView<E> implements OnInit, OnDestroy {

    private _type: E;
    private _document: any;
    private _isSearch: boolean;
    private _itemsAux: Array<E>;
    private _items: Array<E>;
    private _eventsSlots: Events;
    private _database: DataProvider;

    constructor(_type: any) {
        this._type = _type;
        this._document = Reflect.get(_type, "document");
        this._isSearch = false;
        this._itemsAux = [];
        this._items = [];

        let injector = ReflectiveInjector.resolveAndCreate([Events, DataProvider, Platform]);
        this._eventsSlots = injector.get(Events);
        this._database = injector.get(DataProvider);
    }

    /**
     *
     */
    ngOnInit() {
        this.initialize();
    }

    /**
     *
     */
    ngOnDestroy() {
        this._eventsSlots.unsubscribe(Slots.ITEM_SAVED);
    }

    /**
     *
     */
    private initialize() {
        this._eventsSlots.publish(Slots.APP_LOADING);

        this._database.getAll(this._document).then(items => { 
            this.items.concat(items);
            this._eventsSlots.publish(Slots.APP_LOADED);
        }).catch(e => { 
            this._eventsSlots.publish(Slots.APP_LOADED);
            this._eventsSlots.publish(Slots.APP_ERROR, "No se ha podido cargar los datos");
        });
        
        this._eventsSlots.subscribe(Slots.ITEM_SAVED, data => {
            this.add(data);
        });
    }

    /**
     *
     */
    set items(_items: Array<E>) {
        this._items = _items;
    }

    /**
     *
     */
    get items() {
        return this._items;
    }

    /**
     *
     */
    add(o: E): void {
        this.items.push(o);
    }

    /**
     *
     */
    remove(o: E) {
        let index = this.items.indexOf(o);
        this.items.splice(index, 1);
    }

    /**
     *
     */
    get(index: number) {
        return this.items[index];
    }

    /**
     *
     */
    hasItems(): boolean {
        return this._items.length != 0;
    }

    /**
     *
     */
    isSearch(): boolean {
        return this._isSearch;
    }

    /**
     *
     */
    count(): number {
        return this.items.length;
    }

    /**
     *
     */
    toggleSearch(): void {
        this._isSearch = !this._isSearch;

        if (this.isSearch()) {
            this._itemsAux = this._items;
        } else {
            this._items = this._itemsAux;
        }
    }

    /**
     *
     */
    search(text: string): void {
        
    }
}